@@ -1,18 +1,16 @@
 import re
 import gzip
 import numpy as np
-import skbio
 import os
 import time
 import random
 import math
-from Bio import SeqIO
-from Bio import Seq
-from collections import OrderedDict
+from Bio import Seq, SeqIO
+from collections import OrderedDict, Counter
 from pyfaidx import Fasta
-from lib.common.util import logging_init
-import lib.common.sqlite_command as sc
-from lib.common.os import remove_file_name_suffix, cmd_run
+from toolbiox.lib.common.util import logging_init
+import toolbiox.lib.common.sqlite_command as sc
+from toolbiox.lib.common.os import remove_file_name_suffix, cmd_run
 # from lib.common.genome.genome_feature2 import cds_judgment
 
 
@@ -137,14 +135,54 @@ def sub_seq(seq, site_start, site_end, strand, RNA):
     return sub_seq_out
 
 
-def sequence_kmer_frequencies(sequence_input, kmer):
-    s = skbio.Sequence(sequence_input)
-    freqs = s.kmer_frequencies(kmer, overlap=False)
+def iter_kmers(seq, k, overlap=True):
+    """
+    from skbio.Sequence.iter_kmers
+    """
+    if k < 1:
+        raise ValueError("k must be greater than 0.")
+
+    if overlap:
+        step = 1
+        count = len(seq) - k + 1
+    else:
+        step = k
+        count = len(seq) // k
+
+    seq_bytes = np.array([ord(i) for i in seq], dtype='uint8')
+
+    # Optimized path when positional metadata doesn't need slicing.
+    kmers = np.lib.stride_tricks.as_strided(
+        seq_bytes, shape=(k, count), strides=(1, step)).T
+
+    for s in kmers:
+        yield s
+
+
+def kmer_frequencies(seq, k, overlap=True, relative=False):
+    """
+    from skbio.Sequence.kmer_frequencies
+    """
+    kmers = iter_kmers(seq, k, overlap=overlap)
+    freqs = dict(Counter((str(seq) for seq in kmers)))
+
+    if relative:
+        if overlap:
+            num_kmers = len(seq) - k + 1
+        else:
+            num_kmers = len(seq) // k
+
+        relative_freqs = {}
+        for kmer, count in freqs.items():
+            relative_freqs[kmer] = count / num_kmers
+        freqs = relative_freqs
+
     return freqs
 
 
 def sequence_entropy(sequence_input, kmer):
-    freqs = sequence_kmer_frequencies(sequence_input, kmer)
+    freqs = kmer_frequencies(sequence_input, kmer,
+                             overlap=False, relative=False)
     t_num = sum([freqs[i] for i in freqs])
 
     entropy = 0
